@@ -32,93 +32,25 @@ outside of this lifecycle, the send_to_server function can be used to send and
 receive information from the server outside of the default times. Such
 communications should be limited to the body of an on_interact method.
 """
-BACKUP_FILE = ".ok_messages"
-
 from client import config
+from client import exceptions
 from client.models import *
 from client.protocols import *
 from client.utils import auth
 from client.utils import loading
+from client.utils import network
 from client.utils import output
 from datetime import datetime
-from urllib import request, error
-import client
+from urllib import error
 import argparse
-import base64
-import json
+import client
 import multiprocessing
 import pickle
 import sys
-import time
+import logging
 
-def send_to_server(access_token, messages, name, server,
-                   insecure=False):
-    """Send messages to server, along with user authentication."""
-    data = {
-        'assignment': name,
-        'messages': messages,
-    }
-    try:
-        prefix = "http" if insecure else "https"
-        address = prefix + '://' + server + '/api/v1/submission'
-        serialized = json.dumps(data).encode(encoding='utf-8')
-        # TODO(denero) Wrap in timeout (maybe use PR #51 timed execution).
-        # TODO(denero) Send access token with the request
-        address += "?access_token={0}&client_version={1}".format(
-            access_token, client.__version__)
-        req = request.Request(address)
-        req.add_header("Content-Type", "application/json")
-        response = request.urlopen(req, serialized)
-        return json.loads(response.read().decode('utf-8'))
-    except error.HTTPError as ex:
-        # print("Error while sending to server: {}".format(ex))
-        try:
-            if ex.code == 403:
-                response = ex.read().decode('utf-8')
-                response_json = json.loads(response)
-                get_latest_version(response_json['data']['download_link'])
-            #message = response_json['message']
-            #indented = '\n'.join('\t' + line for line in message.split('\n'))
-            #print(indented)
-            return {}
-        except Exception as e:
-            # print(e)
-            # print("Couldn't connect to server")
-            pass
-
-def dump_to_server(access_token, msg_queue, name, server, insecure, staging_queue):
-    while not msg_queue.empty():
-        message = msg_queue.get()
-        staging_queue.put(message)
-        try:
-            if send_to_server(access_token, message, name, server, insecure) != None:
-                staging_queue.get() #throw away successful message
-        except error.URLError as ex:
-            pass
-    return
-
-#####################
-# Software Updating #
-#####################
-
-def get_latest_version(download_link):
-    """Check for the latest version of ok and update this file accordingly.
-    """
-    #print("We detected that you are running an old version of ok.py: {0}".format(VERSION))
-
-    # Get server version
-
-    try:
-        req = request.Request(download_link)
-        response = request.urlopen(req)
-
-        zip_binary = response.read()
-        with open('ok', 'wb') as f:
-            f.write(zip_binary)
-        #print("Done updating!")
-    except error.HTTPError:
-        # print("Error when downloading new version")
-        pass
+BACKUP_FILE = ".ok_messages"
+LOGGING_FORMAT = '%(levelname)-10s | pid %(process)d | %(filename)s, line %(lineno)d: %(message)s'
 
 ##########################
 # Command-line Interface #
@@ -140,6 +72,8 @@ def parse_input():
                         help="unlock tests interactively")
     parser.add_argument('-v', '--verbose', action='store_true',
                         help="print more output")
+    parser.add_argument('--debug', action='store_true',
+                        help="show debug statements")
     parser.add_argument('--insecure', action='store_true',
                         help="uses http instead of https")
     parser.add_argument('-i', '--interactive', action='store_true',
@@ -160,14 +94,19 @@ def parse_input():
                         help="Scores the assignment")
     return parser.parse_args()
 
-def server_timer():
-    """Timeout for the server."""
-    time.sleep(0.8)
-
 def main():
     """Run all relevant aspects of ok.py."""
     args = parse_input()
 
+    logging.basicConfig(format=LOGGING_FORMAT)
+    log = logging.getLogger(__name__)
+    if args.debug:
+        log.setLevel(logging.INFO)
+    else:
+        log.setLevel(logging.ERROR)
+
+    log.info(args)
+
     if args.version:
         print("okpy=={}".format(client.__version__))
         exit(0)
@@ -175,7 +114,8 @@ def main():
     if not args.local and not args.insecure:
         try:
             import ssl
-        except: 
+        except:
+            log.warning('Error importing ssl', stack_info=True)
             sys.exit("SSL Bindings are not installed. You can install python3 SSL bindings or \nrun ok locally with python3 ok --local")
 
 
@@ -183,14 +123,23 @@ def main():
     try:
         print("You are running version {0} of ok.py".format(client.__version__))
         if not args.local:
-            timer_thread = multiprocessing.Process(target=server_timer, args=())
+            timer_thread = multiprocessing.Process(target=network.server_timer,
+                                                   args=())
             timer_thread.start()
+
         cases = {case.type: case for case in core.get_testcases(config.cases)}
-        assignment = loading.load_tests(args.tests, cases)
+        assignment = None
+        try:
+            assignment = loading.load_tests(args.tests, cases)
+        except exceptions.OkException as e:
+            print('Error:', e)
+            exit(1)
 
-        logger = sys.stdout = output.OutputLogger()
+        log.info('Replacing stdout with OutputLogger')
+        output_logger = sys.stdout = output.OutputLogger()
 
-        protocols = [p(args, assignment, logger)
+        log.info('Instantiating protocols')
+        protocols = [p(args, assignment, output_logger, log)
                      for p in protocol.get_protocols(config.protocols)]
 
         messages = dict()
@@ -200,51 +149,49 @@ def main():
         try:
             with open(BACKUP_FILE, 'rb') as fp:
                 file_contents = pickle.load(fp)
-        except IOError as e:
-            # File doesn't exist, so file_contents should be empty
-            pass
-        except EOFError as e:
-            # File is empty, so no messages are inside
-            pass
+                log.info('Loaded %d backed up messages from %s',
+                         len(file_contents), BACKUP_FILE)
+        except (IOError, EOFError) as e:
+            log.info('Error reading from ' + BACKUP_FILE \
+                    + ', assume nothing backed up')
 
         for message in file_contents:
             msg_queue.put(message)
 
         for proto in protocols:
+            log.info('Execute %s.on_start()', proto.name)
             messages[proto.name] = proto.on_start()
         messages['timestamp'] = str(datetime.now())
 
         if not args.local:
             try:
                 access_token = auth.authenticate(args.authenticate)
+                log.info('Authenticated with access token %s', access_token)
+
                 msg_queue.put(messages)
                 staging_queue = multiprocessing.Queue()
                 server_thread = multiprocessing.Process(
-                    target=dump_to_server,
+                    target=network.dump_to_server,
                     args=(access_token, msg_queue, assignment['name'],
-                          args.server, args.insecure, staging_queue))
+                          args.server, args.insecure, staging_queue,
+                          client.__version__, log))
                 server_thread.start()
             except error.URLError as ex:
-                # TODO(soumya) Make a better error message
-                # print("Nothing was sent to the server!")
-                pass
-        
+                log.warning('on_start messages not sent to server: %s', str(e))
+
         interact_msg = {}
 
         for proto in protocols:
+            log.info('Execute %s.on_interact()', proto.name)
             interact_msg[proto.name] = proto.on_interact()
 
         interact_msg['timestamp'] = str(datetime.now())
 
         # TODO(denero) Print server responses.
 
-        # TODO(albert): a premature error might prevent tests from being
-        # dumped. Perhaps add this in a "finally" clause.
-        loading.dump_tests(args.tests, assignment)
-
         if not args.local:
             msg_queue.put(interact_msg)
-            
+
             while timer_thread.is_alive():
                 pass
 
@@ -259,13 +206,22 @@ def main():
             while not staging_queue.empty():
                 dump_list.append(staging_queue.get_nowait())
             with open(BACKUP_FILE, 'wb') as fp:
+                log.info('Save %d unsent messages to %s', len(dump_list),
+                         BACKUP_FILE)
                 pickle.dump(dump_list, fp)
 
+            if len(dump_list) == 0:
+                print("Server submission successful")
+
     except KeyboardInterrupt:
         if timer_thread:
             timer_thread.terminate()
         if server_thread:
             server_thread.terminate()
+    finally:
+        if assignment:
+            log.info('Dump tests for %s to %s', assignment['name'], args.tests)
+            loading.dump_tests(args.tests, assignment, log)
 
 if __name__ == '__main__':
     main()
